package net.es.oscars.policy.config;


import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Properties;

import net.es.oscars.PropHandler;

public class ConfigUtil {
    HashMap<String,String> xpathMap;
    private List<String> stringFields;
    private List<String> numberFields;
    private List<String> booleanFields;
    private List<String> dateFields;
    private HashMap<String,String> actionMap;
    private static HashMap<String,String> namespaceMap;
    
    public ConfigUtil(){
        this.xpathMap = new HashMap<String,String>();
        PropHandler propHandler = new PropHandler("fields.properties");
        this.stringFields = new ArrayList<String>();
        this.numberFields = new ArrayList<String>();
        this.dateFields = new ArrayList<String>();
        this.booleanFields = new ArrayList<String>();
        this.actionMap = new HashMap<String,String>();
        this.loadFields(propHandler, "string", this.stringFields);
        this.loadFields(propHandler, "number", this.numberFields);
        this.loadFields(propHandler, "date", this.dateFields);
        this.loadFields(propHandler, "boolean", this.booleanFields);
        this.loadAction(propHandler);
    }
    
    private void loadAction(PropHandler propHandler) {
        Properties actionProps = propHandler.getPropertyGroup("action", true);
        if(actionProps == null || actionProps.isEmpty()){
            return;
        }
        Enumeration actionEnum = actionProps.keys();
        while(actionEnum.hasMoreElements()){
            String action = (String)actionEnum.nextElement();
            this.actionMap.put(action, actionProps.getProperty(action));
        }
        
    }
    
    private static void loadNamespaces() {
        if(namespaceMap != null){ return; }
        PropHandler propHandler = new PropHandler("fields.properties");
        namespaceMap = new HashMap<String,String>();
        Properties nsProps = propHandler.getPropertyGroup("namespace", true);
        if(nsProps == null || nsProps.isEmpty()){
            return;
        }
        Enumeration nsEnum = nsProps.keys();
        while(nsEnum.hasMoreElements()){
            String ns = (String)nsEnum.nextElement();
            namespaceMap.put(ns, nsProps.getProperty(ns));
        }
        
    }

    private void loadFields(PropHandler propHandler, String type, List<String> fields){
        Properties fieldProps = propHandler.getPropertyGroup("field."+type, true);
        if(fieldProps == null || fieldProps.isEmpty()){
            return;
        }
        Enumeration fieldEnum = fieldProps.keys();
        while(fieldEnum.hasMoreElements()){
            String field = (String)fieldEnum.nextElement();
            xpathMap.put(field, fieldProps.getProperty(field));
            fields.add(field);
        }
    }
    
    public boolean validateDateField(String dateField){
        if(dateField.matches("[+\\-]\\d+ (seconds|minutes|hours|days|weeks|months|years)")){
            //relative date
            return true;
        }else if(dateField.matches("((\\d\\d\\d\\d)|\\*)-((\\d\\d)|\\*)-((\\d\\d)|\\*) ((\\d\\d)|\\*):((\\d\\d)|\\*):((\\d\\d)|\\*)")){
            //absolute date
            return true;
        }
        return false;
    }
    
    /**
     * @return the xpathMap
     */
    public HashMap<String, String> getXPathMap() {
        return this.xpathMap;
    }

    /**
     * @return the stringFields
     */
    public List<String> getStringFields() {
        return this.stringFields;
    }

    /**
     * @return the numberFields
     */
    public List<String> getNumberFields() {
        return this.numberFields;
    }

    /**
     * @return the booleanFields
     */
    public List<String> getBooleanFields() {
        return this.booleanFields;
    }

    /**
     * @return the dateFields
     */
    public List<String> getDateFields() {
        return this.dateFields;
    }

    /**
     * @return the actionMap
     */
    public HashMap<String, String> getActionMap() {
        return this.actionMap;
    }

    /**
     * @return the namespaceMap
     */
    public static HashMap<String, String> getNamespaceMap() {
        loadNamespaces();
        return namespaceMap;
    }
}
